@@ -39,6 +39,13 @@
 #include <KSMTP/LoginJob>
 #include <KSMTP/SendJob>
 
+#include <KGAPI/Account>
+#include <KGAPI/AuthJob>
+#include <KGAPI/AccountManager>
+
+#define GOOGLE_API_KEY QStringLiteral("554041944266.apps.googleusercontent.com")
+#define GOOGLE_API_SECRET QStringLiteral("mdT1DjzohxN3npUUzkENT0gO")
+
 using namespace MailTransport;
 
 class SessionPool
@@ -167,7 +174,7 @@ void SmtpJob::startSmtpJob()
         d->session->open();
     } else {
         if (d->session->state() != KSmtp::Session::Authenticated) {
-            startLoginJob();
+            startPasswordRetrieval();
         }
 
         startSendJob();
@@ -177,12 +184,63 @@ void SmtpJob::startSmtpJob()
 void SmtpJob::sessionStateChanged(KSmtp::Session::State state)
 {
     if (state == KSmtp::Session::Ready) {
-        startLoginJob();
+        startPasswordRetrieval();
     } else if (state == KSmtp::Session::Authenticated) {
         startSendJob();
     }
 }
 
+void SmtpJob::startPasswordRetrieval(bool forceRefresh)
+{
+    if (!transport()->requiresAuthentication() && !forceRefresh) {
+        startSendJob();
+        return;
+    }
+
+    if (transport()->authenticationType() == TransportBase::EnumAuthenticationType::XOAUTH2) {
+        auto promise = KGAPI2::AccountManager::instance()->findAccount(
+            GOOGLE_API_KEY, transport()->userName(), { KGAPI2::Account::mailScopeUrl() });
+        connect(promise, &KGAPI2::AccountPromise::finished,
+                this, [forceRefresh, this](KGAPI2::AccountPromise *promise) {
+                    if (promise->account()) {
+                        if (forceRefresh) {
+                            promise = KGAPI2::AccountManager::instance()->refreshTokens(
+                                GOOGLE_API_KEY, GOOGLE_API_SECRET, transport()->userName());
+                        } else {
+                            onTokenRequestFinished(promise);
+                            return;
+                        }
+                    } else {
+                        promise = KGAPI2::AccountManager::instance()->getAccount(
+                            GOOGLE_API_KEY, GOOGLE_API_SECRET, transport()->userName(),
+                            { KGAPI2::Account::mailScopeUrl() });
+                    }
+                    connect(promise, &KGAPI2::AccountPromise::finished,
+                            this, &SmtpJob::onTokenRequestFinished);
+                });
+    } else {
+        startLoginJob();
+    }
+}
+
+void SmtpJob::onTokenRequestFinished(KGAPI2::AccountPromise *promise)
+{
+    if (promise->hasError()) {
+        qCWarning(MAILTRANSPORT_SMTP_LOG) << "Error obtaining XOAUTH2 token:" << promise->errorText();
+        setError(KJob::UserDefinedError);
+        setErrorText(promise->errorText());
+        emitResult();
+        return;
+    }
+
+    const auto account = promise->account();
+    const QString tokens = QStringLiteral("%1\001%2").arg(account->accessToken(),
+                                                          account->refreshToken());
+    transport()->setPassword(tokens);
+    startLoginJob();
+}
+
+
 void SmtpJob::startLoginJob()
 {
     if (!transport()->requiresAuthentication()) {
@@ -224,8 +282,12 @@ void SmtpJob::startLoginJob()
         }
     }
 
+    if (transport()->authenticationType() == Transport::EnumAuthenticationType::XOAUTH2) {
+        passwd = passwd.left(passwd.indexOf(QLatin1Char('\001')));
+    }
+
     login->setUserName(transport()->userName());
-    login->setPassword(transport()->password());
+    login->setPassword(passwd);
     switch (transport()->authenticationType()) {
     case TransportBase::EnumAuthenticationType::PLAIN:
         login->setPreferedAuthMode(KSmtp::LoginJob::Plain);
@@ -237,7 +299,7 @@ void SmtpJob::startLoginJob()
         login->setPreferedAuthMode(KSmtp::LoginJob::CramMD5);
         break;
     case TransportBase::EnumAuthenticationType::XOAUTH2:
-        login->setPreferedAuthMode(KSmtp::LoginJob::XOAuth);
+        login->setPreferedAuthMode(KSmtp::LoginJob::XOAuth2);
         break;
     case TransportBase::EnumAuthenticationType::DIGEST_MD5:
         login->setPreferedAuthMode(KSmtp::LoginJob::DigestMD5);
@@ -258,10 +320,10 @@ void SmtpJob::startLoginJob()
         login->setEncryptionMode(KSmtp::LoginJob::Unencrypted);
         break;
     case Transport::EnumEncryption::TLS:
-        login->setEncryptionMode(KSmtp::LoginJob::TlsV1);
+        login->setEncryptionMode(KSmtp::LoginJob::STARTTLS);
         break;
     case Transport::EnumEncryption::SSL:
-        login->setEncryptionMode(KSmtp::LoginJob::AnySslVersion);
+        login->setEncryptionMode(KSmtp::LoginJob::SSLorTLS);
         break;
     default:
         qCWarning(MAILTRANSPORT_SMTP_LOG) << "Unknown encryption mode" << transport()->encryption();
@@ -314,6 +376,13 @@ void SmtpJob::slotResult(KJob *job)
         return;
     }
 
+    if (qobject_cast<KSmtp::LoginJob*>(job)) {
+        if (job->error() == KSmtp::LoginJob::TokenExpired) {
+            startPasswordRetrieval(/*force refresh */ true);
+            return;
+        }
+    }
+
     // The job has finished, so we don't care about any further errors. Set
     // d->finished to true, so slaveError() knows about this and doesn't call
     // emitResult() anymore.
