@@ -101,12 +101,12 @@ public:
       Note: The Transport objects become invalid as soon as a change occur, so
       they are only suitable for immediate use.
     */
-    QList<Transport *> transports() const;
+    Q_REQUIRED_RESULT QList<Transport *> transports() const;
 
     /**
       Returns a list of all available transport types.
     */
-    TransportType::List types() const;
+    Q_REQUIRED_RESULT TransportType::List types() const;
 
     /**
       Creates a new, empty Transport object. The object is owned by the caller.
@@ -290,7 +290,6 @@ private:
     TransportManagerPrivate *const d;
 
     Q_PRIVATE_SLOT(d, void slotTransportsChanged())
-    Q_PRIVATE_SLOT(d, void slotWalletOpened(bool success))
 };
 } // namespace MailTransport
 
